
(in-package :om)

;;; A subclass of voiceEditor
;;; analysis-mode is a switch between the 'normal' voice editor and the specific analysis features
;;; selected-segment is a list of indices for the currently selected segments
(defclass SegmentationEditor () 
  ((analysis-mode :accessor analysis-mode :initform t)
   (selected-segment :accessor selected-segment :initform nil)))
(defclass SegmentationPanel () ())

;;; A subclass of voiceEditor
(defclass voiceSegEditor (SegmentationEditor voiceeditor) ())
(defmethod get-editor-class ((self voice-seg)) 'voiceSegEditor)
;;; A subclass of voicePanel
(defclass voiceSegPanel (SegmentationPanel voicepanel) ())
(defmethod get-score-class-panel ((self voiceSegEditor)) 'voiceSegPanel)

;;; A subclass of voiceEditor
(defclass csSegEditor (SegmentationEditor chordseqeditor) ())
(defmethod get-editor-class ((self cs-seg)) 'csSegEditor)
;;; A subclass of chordseqPanel
(defclass csSegPAnel (SegmentationPanel chordseqpanel) ())
(defmethod get-score-class-panel ((self csSegEditor)) 'csSegPanel)


;;; Additional Item in the Menu Bar
(defmethod get-menubar ((self SegmentationEditor)) 
  (append (call-next-method)
          (list (om-make-menu "Analysis" (list  (list :selection (om-new-leafmenu  "Analysis Mode"
                                                                 ;;; switches analysis mode on/off
                                                                 #'(lambda () 
                                                                     (setf (analysis-mode self) (not (analysis-mode self)))
                                                                     (change-text (title-bar self) (if (analysis-mode self)
                                                                                                       "SEGMENTATION/ANALYSIS"
                                                                                                     "VOICE-SEG"))
                                                                     (update-panel (panel self)))
                                                                 "n" t #'(lambda () (analysis-mode self))))
                                                (list 
                                                 (om-new-leafmenu  "Segmentation"
                                                                   #'(lambda () 
                                                                       (om-run-process "segmentation" editor-segmentation self))
                                                                   nil #'(lambda () (analysis-mode self))
                                                                   )))))
          ))


(defun editor-segmentation (editor)
  (setf (segments (object editor)) (pardo-segmentation (real-chords (object editor))))
  ;(auto-segmentation (object editor))
  (update-panel (panel editor)))
                                        

;;; UTILITIES FOR BROWSING THE GRAPHICAL SCORE COMPONENTS
(defmethod real-chords ((self simple-graph-container))
  (loop for sub in (inside self)
        ;when (not (cont-chord-p sub))
        if (subtypep (type-of sub) 'grap-chord) collect sub
        else if (subtypep (type-of sub) 'grap-container) append (real-chords sub)))

(defmethod get-grap-measure ((self grap-ryth-chord))
  (get-grap-measure (parent self)))

(defmethod get-grap-measure ((self grap-group))
  (get-grap-measure (parent self)))

(defmethod get-grap-measure ((self grap-measure)) self)

(defmethod get-grap-measure ((self t)) nil)




(defmethod get-segment-rectangle (segment (grap-container grap-chord-seq))
  (let ((c1 (find (nth (car (car segment)) (real-chords (reference grap-container)))
                  (real-chords grap-container) :test 'equal :key 'reference))
        (c2 (find (nth (car (last (car segment))) (real-chords (reference grap-container)))
                  (real-chords grap-container) :test 'equal :key 'reference)))
    (when (and c1 c2)
      (list (- (first (rectangle c1)) 4) 
            (- (second (rectangle grap-container)) 4)
            (+ (third (rectangle c2)) 4) 
            (+ (fourth (rectangle grap-container)) 16)))))

(defmethod get-segment-rectangle (segment (grap-container grap-voice))
  (let ((c1 (find (nth (car (car segment)) (real-chords (reference grap-container)))
                  (real-chords grap-container) :test 'equal :key 'reference))
        (c2 (find (nth (car (last (car segment))) (real-chords (reference grap-container)))
                  (real-chords grap-container) :test 'equal :key 'reference)))
    (when (and c1 c2)
      (list (- (first (rectangle c1)) 4) 
            (- (min (second (rectangle (get-grap-measure c1))) (second (rectangle (get-grap-measure c2)))) 4)
            (+ (third (rectangle c2)) 4) 
            (+ (max (fourth (rectangle (get-grap-measure c1))) (fourth (rectangle (get-grap-measure c2)))) 16)
            ))))


;;; KEYBOARD EVENT
(defmethod handle-key-event ((self SegmentationPanel) key)
  (if (analysis-mode (editor self))
      ;;; active only if analysis mode is on..
      ;;; 'S'
      (cond ((equal key #\s)
             (if (selected-segment (editor self))
                 (progn
                   ;;; If a segment is seleted, computes (or recomputes) the analysis
                   (compute-segments (object (editor self)) (selected-segment (editor self)))
                   (update-panel self))

               (let* ((sel-chords (selection? self))
                      (real-chords (real-chords (object (editor self))))
                      (positions (mapcar #'(lambda (c) 
                                             (position c real-chords :test 'equal)) 
                                         sel-chords)))
                 ;;; If chords are selected, create a new segment
                 (when positions
                   (new-segment (object (editor self)) positions))
                 (update-panel self)
                 )))
            ((equal key #\m)
             (print (get-minimal-segments (real-chords (object (editor self))))))

            ((equal key :om-key-delete)
             ;;; remove segment
             (when (selected-segment (editor self))
               (remove-segments (object (editor self)) (selected-segment (editor self)))
               (setf (selected-segment (editor self)) nil)
               (update-panel self)
               ))
            (t (call-next-method)))
    (call-next-method)))


;;; MOUSE CLICK HANDLER
(defmethod om-score-click-handler ((self SegmentationPanel) where double-click-p)
   (if (analysis-mode (editor self))
       (let* ((voice-seg (object (editor self)))
              (selectedpos nil)
              (selecteditem (loop for item in (segments voice-seg) 
                                  for i = 0 then (+ i 1)
                                  while (not selectedpos) do 
                                  (let ((rect (get-segment-rectangle item (graphic-obj self))))
                                    (when (and rect (om-point-in-rect-p where (om-make-rect (car rect) (cadr rect) (caddr rect) (cadddr rect))))
                                      (setf selectedpos i)
                                      (return item))))))
         (setf (selected-segment (editor self)) (if selectedpos (list selectedpos) nil))
         (if selectedpos
             ;(if double-click-p 
             ;    (modify-segment (object (editor self)) selectedpos where)
               (update-panel self)
           ;)
           (call-next-method)))
         (call-next-method)))

;;; menu context : change the chord from the different available possibilities
(defmethod om-get-menu-context ((self SegmentationPanel))
  (if (analysis-mode (editor self))
      (when (selected-segment (editor self))
        (let ((options (cadr (nth (car (selected-segment (editor self))) (segments (object (editor self)))))))
          (list (mapcar #'(lambda (op) 
                      (let ((str (if (stringp op) op (format nil  "~a (~d)" (cadr op) (car op) ))))
                        (om-new-leafmenu str
                                         #'(lambda () 
                                             (correct-chord (object (editor self)) 
                                                            (car (selected-segment (editor self))) 
                                                            str)
                                             (update-panel self)
                                             ))))
                  (first-n options 5))
          (mapcar #'(lambda (base)
                      (om-new-menu base
                             (om-new-leafmenu (string+ base "M") 
                                              #'(lambda () (correct-chord (object (editor self)) 
                                                                          (car (selected-segment (editor self))) (string+ base "M")) 
                                                  (update-panel self)))
                             (om-new-leafmenu (string+ base "Dom7") 
                                              #'(lambda () (correct-chord (object (editor self)) 
                                                                          (car (selected-segment (editor self))) (string+ base "Dom7"))
                                                  (update-panel self)))
                             (om-new-leafmenu (string+ base "m") 
                                              #'(lambda () (correct-chord (object (editor self)) 
                                                                          (car (selected-segment (editor self))) (string+ base "m")) 
                                                  (update-panel self)))
                             (om-new-leafmenu (string+ base "d7") 
                                              #'(lambda () (correct-chord (object (editor self)) 
                                                                          (car (selected-segment (editor self))) (string+ base "d7")) 
                                                  (update-panel self)))
                             (om-new-leafmenu (string+ base "hd7") 
                                              #'(lambda () (correct-chord (object (editor self)) 
                                                                          (car (selected-segment (editor self))) (string+ base "hd7")) 
                                                  (update-panel self)))
                             (om-new-leafmenu (string+ base "dim") 
                                              #'(lambda () (correct-chord (object (editor self)) 
                                                                          (car (selected-segment (editor self))) (string+ base "dim")) 
                                                  (update-panel self))))
                      '("C" "C#" "D" "D#" "E" "F" "F#" "G" "G#" "A" "A#" "B"))) 
           )))
    (call-next-method)))
          
;;; MODELO MENU CONTEXTUAL :
;  (list (om-new-menu "XXX"
;                     (om-new-leafmenu "a" #'(lambda () (print "a")))
;                     (om-new-leafmenu "a" #'(lambda () (print "a"))))
;        (list (om-new-leafmenu "b" #'(lambda () (print "a")))
;              (om-new-leafmenu "c" #'(lambda () (print "a"))))))    


;;; SPECIFIC DRAWING OPERATIONS 
(defmethod draw-view-contents ((self SegmentationPanel))
  (call-next-method)
  (let* ((x0  (om-h-scroll-position self))
         (y0  (om-v-scroll-position self))
         (size (staff-size self))
         (deltax (get-key-space self))
         (deltay (round (* size (score-top-margin self)))))
    (om-with-focused-view self
      (om-with-fg-color self (om-make-color 0.4 0.3 0.7)  ;;; change this (or copy around specific parts of the code) to set another color
      (when (analysis-mode (editor self)) 
        (om-with-line-size 3 
          (om-draw-rect x0 y0 (- (w self) 16) (- (h self) 16)))   ;;; a rectangle highlight for analysis mode
        (when (graphic-obj self)
          (om-with-clip-rect self (om-make-rect (+ x0 (- deltax (round size 2))) y0  (+ x0 (w self)) (+ y0 (h self)))
            (draw-analysis-segments (graphic-obj self) (segments (object (editor self))) (staff-sys self) size (selected-segment (editor self)))
            )
          ))))))


;;; DRAW SEGMENTS
(defmethod draw-analysis-segments ((self grap-container) segments system size selection)
  (om-with-font *om-default-font2b* 
   (loop for item in segments 
         for i = 0 then (+ i 1) do
         (let ((rect (get-segment-rectangle item self))
               (acc (cadr item)))
           (when rect
             ;(om-draw-rect (nth 0 rect) (nth 1 rect) (- (nth 2 rect) (nth 0 rect)) (- (nth 3 rect) (nth 1 rect))) 
             (om-with-line-size 2
                (om-draw-line (nth 0 rect) (nth 1 rect) (nth 0 rect) (nth 3 rect))) 
             (when (member i selection)
               ;;; highlight if selected
               (om-with-hilite-foreground
                 (om-fill-rect (nth 0 rect) (nth 1 rect) (- (nth 2 rect) (nth 0 rect)) (- (nth 3 rect) (nth 1 rect)))))
             (when acc
               ;;; analysis data
               (om-draw-string (+ (nth 0 rect) 4)
                               (- (nth 3 rect) 1)
                               (if (stringp acc) acc ;;; data = string
                                 (if (listp acc) 
                                     (if (stringp (car acc)) (car acc) ;;; data = list of strings (draw the first one)
                                       (if (and (listp (car acc)) (stringp (cadr (car acc))))
                                           (cadr (car acc)) ;;; data = list of (weight string) : draw the first string
                                         ""))))))
             )))))



    
    
