(in-package :om)


;;; CONVERSION MIDIFILE=>CS_SEG
(defmethod* objFromObjs ((self midifile) (type cs-seg))
  (let ((newcs (make-instance (type-of type)))
        (midilist (sort (loop for item in (mf-info-mc self)
                              append item) '< :key 'second))
        (accs (sort (mat-trans (multiple-value-list (get-mf-lyrics self))) '< :key 'cadr))) 
    (setQValue newcs 1000 :recursive nil)
    (setf (inside newcs) nil)
    (setf (inside newcs) (make-quanti-chords midilist 0))
    (adjust-extent newcs)
    (QNormalize newcs)
    (setf (segments newcs) (build-segmentation newcs accs))
    newcs
    ))

;;; BUILD FORMATTED SEGMENTS FROM MIDI LYRICS INFO
(defun build-segmentation (cseq accs)
  (let ((onsets (butlast (lonset cseq)))
        (segments nil))
    (remove nil
            (loop for acc in accs 
                  for a = 1 then (+ a 1) 
                  collect
                  (let* ((first-pos (position (cadr acc) onsets :test '<=))
                         (seg (when first-pos (list first-pos)))
                         (next-t (cadr (nth a accs))))
                    (when seg 
                      (if next-t
                          (loop for i = (1+ first-pos) then (+ i 1)
                                while (and (< (nth i onsets) next-t) (< i (length onsets)))
                                do (push i seg))
                        (loop for i = (1+ first-pos) then (+ i 1)
                                while (< i (length onsets))
                                do (push i seg))))
                    (list (reverse seg) (list (list nil (car acc)))))))))


;;; METHODS FOR MIDI EXPORT
(defmethod* PrepareToPlay ((player t) (self cs-seg) at &key approx port interval voice)
   (call-next-method)
   (save-segments-as-lyrics self))

(defmethod* PrepareToPlay ((player t) (self voice-seg) at &key approx port interval voice)
   (call-next-method)
   (save-segments-as-lyrics self))

(defmethod save-segments-as-lyrics ((self segmented-object))
 (setf port (verify-port nil))
 (loop for seg in (segments self) do
       (when (nth (car (car seg)) (real-chords self))
       (let ((event (om-midi-new-evt (om-midi-get-num-from-type "Lyric"))))
         (if (not event)
             (om-beep-msg "MidiShare could not create a Lyric event")
           (let ((text (if (stringp (car (cadr seg))) (car (cadr seg))
                         (if (listp (car (cadr seg))) (cadr (car (cadr seg)))
                           ""))))
             (om-midi-evt-set event :chan 0)			
             (om-midi-evt-set event :port port)
             (om-midi-evt-set event :ref 0)
             (om-midi-evt-set event :date (offset->ms (nth (car (car seg)) (real-chords self))))
             (om-midi-evt-set event :text text)
             (om-midi-seq-add-evt *playing-midi-seq* event))))
       )))
     