(in-package :om)

(defmethod! midip-Crel (pitches)
  (mapcar #'(lambda (x) (mod x 12)) pitches)
)

(defmethod! midic-Crel (pitches)
  (midip-Crel (mapcar #'(lambda (x) (/ x 100)) pitches))
)

(defmethod! extract-pitches ((v sequence*) pos)
 (flat (mapcar #'(lambda (p) (midic-Crel (lmidic (nth p (get-chords v))))) pos))
)

(setf chord-extensions '(((0 4 7) "M") ((0 4 7 10) "Dom7") ((0 3 7) "m") ((0 3 6 9) "d7") ((0 3 6 10) "hd7") ((0 3 6) "dim")))

(setf chord-roots '((0 "C") (1 "C#") (2 "D") (3 "D#") (4 "E") (5 "F") (6 "F#") (7 "G") (8 "G#") (9 "A") (10 "A#") (11 "B")))

( defun distl ( m N )
   ( cond
      ( ( null N ) nil )
      ( t ( cons 
        ( list
          (midip-Crel (mapcar #'(lambda (x) (+ (car m) x)) (car (car N))))
          (concatenate 'string (cadr m) (cadr (car N))))
        ( distl m ( cdr N ))))
   )
)

( defun cartesian ( M N )
   ( cond
      ( ( null M ) nil )
      ( t  ( append ( distl ( car M ) N )
                    ( cartesian ( cdr M )  N )
           )
      )
   )
)

(setf chord-profiles (cartesian chord-roots chord-extensions))

(defmethod! compute-chord-weight (c n)
  (let (notesinchord notesmissing incount outcount missingcount)
    (setf notesinchord (mapcar #'(lambda (x) (count x c)) n))
    (setf notesmissing (mapcar #'(lambda (x) (count x n)) c))
    (setf incount (count 1 notesinchord))
    (setf outcount (count 0 notesinchord))
    (setf missingcount (count 0 notesmissing))
    (- incount outcount missingcount))
)

; (weight-chord (flat '((2 5) (4) (11 3) (7) (9 2) (10))))

(defmethod! weight-chord (c)
  ;(print c)
  (sort (mapcar #'(lambda (profile) (list (compute-chord-weight (car profile) c) (cadr profile))) chord-profiles) #'> :key #'first)
)

(defmethod! weight-chords (cseq &optional (nchords 5))
  (mapcar #'(lambda (c) (first-n (weight-chord c) nchords)) cseq)
)


(defmethod! get-minimal-segments (chords)
  (let ((segments (loop for c in chords collect
                 ; sorts the notes of the chord, just in case some notes
                 ; have different offsets
                 (sort (loop for n in (inside c) collect
                          (list (car (midic-Crel (list (midic n))))
                                (+ (offset->ms c) (offset->ms n))
                                (+ (+ (offset->ms c) (offset->ms n)) (dur n)))) '< :key 'second))))
    ;(print segments)
    (loop for i from 0 upto (- (list-length segments) 2) do
          (let* ((currentseg (nth i segments))
                 (nextseg (nth (1+ i) segments))
                 (nextoffset (second (car nextseg))))
            (loop for j from 0 upto (1- (list-length currentseg)) do
                  (let ((n (nth j currentseg)))
                    (when (> (third n) nextoffset) ; when a note overlaps with the next segment it is divided in two
                      (setf (nth (1+ i) segments)
                            (append (nth (1+ i) segments) (list (list (first n) nextoffset (third n)))))
                      (setf (nth j currentseg)
                            (list (first n)
                                  (second n)
                                  (1- nextoffset))))))))
    segments)
)

(defmethod! extract-minimal-segments ((v sequence*) pos)
 (let ((segments (get-minimal-segments (real-chords v))))
   (mapcar 'first (flat (mapcar #'(lambda (p) (nth p segments)) pos) 1)))
)

(defmethod! pardo-segmentation (chords)            
 (let* ((minimalseg (get-minimal-segments chords))
        (pos -1)
        (segments (loop for s in minimalseg collect
                        (list (list (incf pos)) (mapcar 'first s) (first-n (weight-chord (mapcar 'first s)) 5))))
        (result nil))
   (print segments)

   (while segments
     (print (length segments))
     (if (second segments)
         (let* ((s1 (first segments))
                (s2 (second segments))
                (w1 (caar (third s1)))
                (w2 (caar (third s2)))
                (pitches (append (second s1) (second s2)))
                (newseg (list (append (first s1) (first s2)) pitches (weight-chord pitches))))
           (if (> (car (car (third newseg))) (+ w1 w2))
               (progn (pop segments) (pop segments) (push newseg segments))
               (let ((s (pop segments)))
                 (push (list (first s) (third s)) result))
           ))
       (let ((s (pop segments)))
         (push (list (first s) (third s)) result))))
   (print (reverse result))))
