(in-package :om)

(defclass! segmented-object ()
    ((analysis-segments :accessor analysis-segments :initform nil :initarg :analysis-segments :documentation "segments and analysis data")))

;;; A VOICE WITH A LIST OF SEGMENTS+ANALYSIS DATA
(defclass! voice-seg (voice segmented-object)
  ()
  (:icon 900)
  (:documentation ""))

(defclass! cs-seg (chord-seq segmented-object)
  ()
  (:icon 900)
  (:documentation ""))


(defmethod* Objfromobjs ((Self voice) (Type voice-seg))
   (make-instance (class-of Type) :tree (tree self)
                  :chords (chords self)
                  :tempo (tempo self) :legato (legato self) :ties (ties self)))

(defmethod* Objfromobjs ((Self voice-seg) (Type voice-seg))
   (clone self))


;;; ADD A NEW SEGMENT IN THE VOICE-SEG INCLUDING CHORDS AT 'positions'
(defmethod new-segment ((self segmented-object) positions)
  (let* ((seglist (sort (copy-list (segments self)) '< :key #'(lambda (item) (list-min (car item)))))
         (newpositions (sort positions '<))
         (pmin (car newpositions)) (pmax (car (last newpositions)))
         (seg-position (or (position (car newpositions) seglist :test '<= :key 'caar)
                           (length seglist)))
         (pitches (extract-minimal-segments self positions))
         (weight-chord-list (weight-chord pitches))) 

    (setf (segments self) 
          (remove nil (append 
                       (loop for i from 0 to (1- seg-position) collect
                             (let ((np (remove-if #'(lambda (item) (>= item pmin)) (car (nth i seglist)))))
                               (if np (list np (cadr (nth i seglist))))))
                       (list (list newpositions weight-chord-list))
                       (loop for i from seg-position to (1- (length seglist)) collect
                             (let ((np (remove-if #'(lambda (item) (<= item pmax)) (car (nth i seglist)))))
                               (if np (list np (cadr (nth i seglist))))))
                       )))))

;;; SETS THE DATA IN THE SEGMENTS AT 'position' given their respective contents 
(defmethod compute-segments ((self segmented-object) positions)
  (loop for seg in positions do 
        (let* ((c-positions (car (nth seg (segments self))))
               (pitches (extract-minimal-segments self c-positions)))
          (setf (nth seg (segments self))
                (list c-positions
                      (weight-chord pitches))))))

;;; REMOVES A SEGMENT FROM THE VOICE-SEG
(defmethod remove-segments ((self segmented-object) positions)
   (setf (segments self) 
         (remove nil (loop for item in (segments self)
                           for i = 0 then (+ i 1) collect 
                           (if (member i positions :test '=) nil
                             item)))))


;;; CHANGES THE CHORD FROM A LIST OF POSSIBLE SOLUTIONS 
(defmethod correct-chord ((self segmented-object) position chord)
  (let* ((seg (nth position (segments self)))
         (init-c (find chord (cadr seg) :test 'string-equal :key 'cadr)))
    (setf (nth position (segments self))
          (if init-c
              (list (car seg)
                    (cons init-c (remove chord (cadr seg) :test 'string-equal :key 'cadr)))
              (list (car seg)
                    (cons (list 1 chord) (cadr seg)))))
    ))



(defmethod! auto-segmentation ((self segmented-object))
    :icon 300
    (let* ((chords (get-chords self))
           (len (length chords))
           (i 0))
      (show-message-win "Segmentation...")
      (setf (segments self) nil)
      (loop while (< i (length chords))
            do (let* ((n (om-random 3 10))
                      (nums (arithm-ser i (min (+ i (1- n)) (1- len)) 1)))
                 (change-message-win (format nil "Segmentation...  ~d%" (round (* 100 (/ i len)))))
                 (new-segment self nums)
                 (setf i (+ i n))))
      (close-message-win)
      (segments self)))


;;;; OM BOX
(defmethod! voice-segmentation ((self voice))
    :icon 300
    (let ((rep (objfromobjs self (make-instance 'voice-seg))))
      (auto-segmentation rep)
      rep))

(defmethod! voice-segmentation ((self chord-seq))
    :icon 300
    (let ((rep (objfromobjs self (make-instance 'cs-seg))))
      (auto-segmentation rep)
      rep))
 
 
      


     
